package com.jessebrault.di;

import jakarta.inject.Provider;
import jakarta.inject.Singleton;

import static com.jessebrault.di.BindingUtil.toLazySingleton;

public final class SingletonScopeHandler implements ScopeHandler<Singleton> {

    private final Registry owner;

    public SingletonScopeHandler(Registry owner) {
        this.owner = owner;
    }

    @Override
    public <T> Binding<T> onScopedDependencyRequest(
            Singleton annotation,
            Class<T> dependencyClass,
            RegistryObjectFactory objectFactory
    ) {
        final Binding<T> potentialBinding = this.owner.getBinding(dependencyClass);
        return switch (potentialBinding) {
            case ClassBinding<T>(Class<T> from, Class<? extends T> to) -> {
                this.owner.bind(from, toLazySingleton(() -> objectFactory.createInstance(to)));
                yield this.owner.getBinding(from);
            }
            case ProviderBinding<T>(Class<T> from, Provider<? extends T> provider) -> {
                this.owner.bind(from, toLazySingleton(provider::get));
                yield this.owner.getBinding(from);
            }
            case SingletonBinding<T> singletonBinding -> singletonBinding;
            case LazySingletonBinding<T> lazySingletonBinding -> lazySingletonBinding;
            case null -> {
                this.owner.bind(dependencyClass, toLazySingleton(() -> objectFactory.createInstance(dependencyClass)));
                yield this.owner.getBinding(dependencyClass);
            }
        };
    }

    @Override
    public void reset() {
        throw new UnsupportedOperationException("Cannot reset the Singleton scope!");
    }

}
